from ..services import printer, client, util
from ..const import COMMAND
from blessings import Terminal
import time


def list(all=False, state=None, order_by='JobId', desc='', limit=20, watch=False):

    if watch:
        try:
            printer.print_watching_tip(3)

            opt = list_jobs(state, order_by, desc, limit)
            n = print_jobs(all, opt)

            time.sleep(3)
            term = Terminal()
            while True:
                with term.location(0, term.height - n - 2):

                    opt = list_jobs(state, order_by, desc, limit)

                    print(term.clear_eos + term.move_up)

                    printer.print_watching_tip(3)
                    n = print_jobs(all, opt)

                    time.sleep(3)
        except KeyboardInterrupt as e:
            print('KeyboardInterrupt')
    else:
        opt = list_jobs(state, order_by, desc, limit)
        print_jobs(all, opt)
        printer.tip('Type "%s job <job_id>" to get job detail' % COMMAND)




def list_jobs(state=None, order_by='JobId', desc='', limit=20):
    job_list = client.list_jobs()
    total = len(job_list)

    # format Date
    job_list = printer.format_date_fields(job_list,['StartTime','EndTime','CreateTime','CreationTime'], util.JOB_KEYS)
    t = job_list


    # filter states
    if state != None:
        t = util.filter_list(t, {'State': state.split(',')})
        total = len(t)


    # order by
    t = util.order_by(t, order_by.split(','), False if (desc and desc.lower()=='false') else True)

    # limit
    if limit>= 0:
        t = t[:int(limit)]

    # calc Progress
    t = util.calc_job_progress(t)

    return {'jobs': t, 'limit':limit, 'total': total}

def print_jobs(all, opt):
    n = 0
    n += printer.print_title('Jobs (limit %d rows):' % opt['limit'])

    if all:
        n+=printer.print_arr(opt['jobs'])
    else:
        n+=printer.print_arr(opt['jobs'], util.JOB_COLS)

    n += printer.total(opt['total'])

    return n


def transform_state(state=''):
    return {
        'waiting': 'Waiting',
        'running': 'Running',
        'failed': 'Failed',
        'finished': 'Finished',
        'stopped': 'Stopped'
    }.get(state.lower())

def transform_limit(limit=10):

    try:
        return int(limit)
    except:
        raise Exception('--limit must be a Positive interger')
