import json
from terminal import *
from .util import *

from time import strftime,localtime
from . import dag,validator
import random

IND = 'No.'

def print_title(s):
    print('\n%s' % bold(magenta(s)))
    return 2


def print_arr(arr, cols=[], show_no=True):
    '''
    arr: result rows
    cols: shown fields. example: ['ImageId', 'ImageName', ...]
    '''
    if len(arr)==0:
        print('Empty array')
        return 1

    if len(cols)==0:
        return __print_all_fields(arr)
    else:
        return __print_col_fields(arr, cols, show_no)



def print_task_desc(task_name, task_desc):
    t = []

    t.append('%s' % ('-'*90))

    t.append('\n%s: %s' % (white('TaskName'), task_name) )
    t.append('\t%s: %s' % (white('ImageId'), task_desc.get('ImageId')) )
    t.append('\t%s: %s' % (white('InstanceCount') , task_desc.get('InstanceCount')) )

    t.append('\n%s: %s' % (white('ProgramType') ,task_desc.get('ProgramType')))
    t.append('\t%s: %s' % (white('ProgramName') ,task_desc.get('ProgramName')))
    t.append('\t%s: %s' % (white('ProgramArguments') ,task_desc.get('ProgramArguments') ))

    t.append('\n%s: %s' % (white('PackageUri') ,task_desc.get('PackageUri') ))
    t.append('\n%s: %s' % (white('StdoutRedirectPath') ,task_desc.get('StdoutRedirectPath') ))
    t.append('\n%s: %s' % (white('StderrRedirectPath') ,task_desc.get('StderrRedirectPath') ))

    t.append('\n%s: %s' % (white('Timeout') ,task_desc.get('Timeout') ))

    if task_desc.get('ResourceDescription'):
        t.append('\t%s: %s' % (white('CPU'), task_desc['ResourceDescription'].get('Cpu') ))
        t.append('\t%s: %s' % (white('Memory'), task_desc['ResourceDescription'].get('Memory') ))

    t.append('\n%s:' % (white('EnvironmentVariables')) )
    envs = task_desc.get('EnvironmentVariables')
    if envs:
        for k in envs:
            t.append('\n  %s : %s' % (k, envs.get(k)))

    t.append('\n%s: %s' % (white('OssMappingLock'), 'true' if task_desc.get('OssMappingLock') else 'false' ))
    t.append('\t%s: %s' % (white('OssMappingLocale'), task_desc.get('OssMappingLocale') ))

    t.append('\n%s:' % (white('OssMapping')) )
    oss_mapping = task_desc.get('OssMapping')
    if oss_mapping:
        for k in oss_mapping:
            t.append('\n  %s : %s' % (k, oss_mapping.get(k)))
    t.append('\n')
    t.append('%s' % white('-'*90))

    s = ''.join(t)
    print(s)
    return len(s.split('\n'))

def print_instance_blocks(arr):
    t=[]
    c=0
    for item in arr:
        c+=1
        ins = to_dict(item, INSTANCE_COLS)

        id = str(ins['InstanceId'])
        color_state = get_color_state(ins['State'])
        t.append('%s:%s' % (id, color_state) )
        t.append('\n' if (c%7==0) else '\t')

    s = ''.join(t)
    print(s)
    return len(s.split('\n'))

def __print_col_fields(arr, cols, show_no=True):
    """
    only print the expected columns in each item
    """
    # calc
    col_max_len_map = {}
    for c in cols:
        col_max_len_map[c] = len(c)

    for item in arr:
        for c in cols:
            v = item.get('ProgressTxt') if c=='Progress' else item.get(c)
            col_max_len_map[c] = max( len(__to_str(v)),  col_max_len_map[c] )

    if show_no:
        col_max_len_map[IND] = max( len(IND), len(str(len(arr)) ) )

    # print cols
    t = []
    t2 = []
    t3 = []

    if show_no:
        t2.append('%s%s' % (IND, __fit_len(IND, col_max_len_map[IND])))
        t3.append('-' * col_max_len_map[IND] )

    for c in cols:
        t2.append( '%s%s' % (c,  __fit_len(c, col_max_len_map[c])))
        t3.append('-'*col_max_len_map[c])

    t.append('| %s |' % ' | '.join(t2))
    t.append('| %s |' % ' | '.join(t3))


    # print data
    for (i,item) in enumerate(arr):
        t2 = []

        if show_no:
            t2.append('%s%s' % (i+1 , __fit_len(i+1, col_max_len_map[IND])));

        for c in cols:

            v = __to_str('' if item.get(c)==None  else item[c])

            if c=='State':
                v2 = get_color_state(v)
                t2.append( '%s%s' % (v2 , __fit_len(v, col_max_len_map[c])))

            elif c=='Progress':
                t2.append('%s%s' % (item.get('Progress'),  __fit_len(item.get('ProgressTxt'), col_max_len_map[c])))

            elif c=='StoppedTask':

                v2 = get_color_state('Stopped', v) if item[c] > 0 else v
                t2.append( '%s%s' % (v2 , __fit_len(v, col_max_len_map.get(c))))

            else:
                t2.append( '%s%s' % (v , __fit_len(v, col_max_len_map.get(c))) )

        t.append('| %s |' % ' | '.join(t2))

    s = '\n'.join(t)
    print(s)
    return len(s.split('\n'))


def __print_all_fields(arr):
    """
    print all fields in each item
    """
    cols = []
    for k in arr[0].keys():
        cols.append(k)

    t = []
    for (i,item) in enumerate(arr):
        t2=[]
        for c in cols:
            v = json.dumps(item.get(c)) if type(item.get(c)) != str else item.get(c)
            t2.append(' %s: %s' % (c, (get_color_state(v) if c=='State' else v)) )
        t.append('%s' % '=' * 20  )
        t.append('%d.---' % (i+1) )
        t.append('%s' % '\n'.join(t2))
    print('\n'.join(t))
    return len(t)



def print_watching_tip(n):
    title = white('Auto refresh in %d seconds' % n)
    locale_time = strftime('%Y-%m-%d %H:%M:%S',localtime())

    print('%s(%s)%s' %( blue_bg(title) , yellow(locale_time) , yellow_bg('.' * random.randint(1,10))  ))
    return 1



def print_job_desc(obj):
    t = []

    t.append('%s: %s' % (white('JobName'), obj.get('JobName') ))
    t.append('\t%s: %s' % (white('JobTag'), obj.get('JobTag')))
    t.append('\t%s: %s' % (white('Priority'), obj.get('Priority')))

    s = ''.join(t)
    print(s)
    return len(s.split('\n'))

def print_deps(deps, matric=None):
    n = print_title('DAG:')
    n += dag.draw(deps, matric)
    return n

def warn(s):
    print(red_bg(s))
    return len(s.split('\n'))

#########################
def done(s=None):
    s = '%s' % (s or 'Done')
    print(green(s))
    return len(s.split(','))

def total(c):
    return done('Total %d rows.' % c)

def print_count(c):
    return done('Found %d rows.' % c)

def tip(s):
    print(white('\n  %s\n' % s))
    return 3

def __to_str(s):
    return s if isinstance(s, STRING) else json.dumps(s)

def __fit_len(s, spacerLen):
    n = spacerLen - len(__to_str(s))
    return ' ' * n
